<?php
/*
NetworX - open-source social networks platform
Copyright (C) 2009 SocialABC, Inc. http://www.socialabc.com
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program in a file called LICENSE; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/ 

include('includes/application_top.php');

if (!Session::isCurrentSessionLoggedIn())
{
	cmn_unauthorized_action();
}

$group_id = isset($_GET['group_id']) ? $_GET['group_id'] : ( isset($_POST['group_id']) ? $_POST['group_id'] : -1 );
$group = new Group($group_id);

if ( ($group->GroupID == -1) || (!$group->hasMember($user->UserID)) )
{
	cmn_unauthorized_action();
}

$template->setGlobalVariable('GroupID', $group_id);

// only administrator has got rights to modify todos
if ($group->userIsAdmin($user->UserID))
{
	if ($_POST['submitted'] == 'true')
	{
		$error = false;

		foreach ($_POST['TodoName'] as $todo_name)
		{
			if (empty($todo_name))
			{
				$error = true;
				$template->touchBlock('TodoName_is_empty');
				break;
			}
		}
		if (false === $error)
		{
			foreach ($_POST['TodoDescription'] as $todo_description)
			{
				if (empty($todo_description))
				{
					$error = true;
					$template->touchBlock('TodoDescription_is_empty');
					break;
				}
			}
		}

		if (false === $error)
		{
			if ($_POST['action'] == 'save_todo')
			{
				$todo_names = $_POST['TodoName'];
				$todo_descriptions = $_POST['TodoDescription'];
				$todo_statuses = $_POST['TodoStatus'];

				Todo::clearGroupTodos($group_id);
				for ($i = 0; $i < count($todo_names); $i++)
				{
					$todo = new Todo();
					$todo->GroupID = $group_id;
					$todo->TodoName = $todo_names[$i];
					$todo->TodoDescription = $todo_descriptions[$i];
					$todo->TodoStatus = ($todo_statuses[$i] == TODO_STATUS_COMPLETED) ? TODO_STATUS_COMPLETED : TODO_STATUS_IN_PROGRESS;
					$todo->saveToDB();
				}
			}
		}

		if ($_POST['action'] == 'assign_to_member' && !empty($_POST['member_id']))
		{
			$todo_id = $_POST['todo_id'];
			$todo = new Todo($todo_id);
			$member_id = $_POST['member_id'];
			$todo->UserID = $member_id;
			$todo->saveToDB();

			Message::sendUserMessage($member_id, $user->UserID, TODO_NOTIFICATION_MESSAGE_SUBJECT,  TODO_NOTIFICATION_MESSAGE_TEXT);
		}
	}
	elseif ($_GET['action'] == 'delete_todo' && !empty($_GET['todo_id']))
	{
		$todo_id = $_GET['todo_id'];
		$todo = new Todo($todo_id);
		$todo->delete();
	}
	elseif ($_GET['action'] == 'mark_as_completed' && !empty($_GET['todo_id']))
	{
		$todo_id = $_GET['todo_id'];
		$todo = new Todo($todo_id);
		$todo->TodoStatus = TODO_STATUS_COMPLETED;
		$todo->saveToDB();
	}
}

$todos = Todo::getGroupTodos($group_id);
foreach ($todos as $key => $todo)
{
	$todo_id = $todo->TodoID;
	$todo_name = $todo->TodoName;
	$todo_description = $todo->TodoDescription;
	$todo_status = $todo->TodoStatus;

	$template->setVariable(array(
		'TodoID' => $todo_id,
		'TodoName' => $todo_name,
		'TodoDescription' => $todo_description,
	));

	if ($todo_status == TODO_STATUS_COMPLETED)
	{
		$template->setVariable('TodoStatus', TODO_STATUS_COMPLETED);
		$template->touchBlock('completed');
	}
	elseif ($todo_status == TODO_STATUS_IN_PROGRESS)
	{
		if ($group->userIsAdmin($user->UserID))
		{
			$template->setVariable('TodoID', $todo->TodoID);
			$template->parse('in_progress');
		}
	}

	if ($group->userIsAdmin($user->UserID))
	{
		$template->setVariable('TodoID', $todo->TodoID);
		$template->parse('assign_link');
		$template->setVariable('TodoID', $todo->TodoID);
		$template->parse('admin_links');
	}
	$template->setVariable('TodoID', $todo->TodoID);
	$template->parse('todo');
}

if ($group->userIsAdmin($user->UserID))
{
	$template->touchBlock('add_link');
}

$template->setVariable('Entity', 'group');
$template->setVariable('EntityID',   $group_id);
$template->parse('back_link');

include('includes/application_bottom.php');