<?php
/*
NetworX - open-source social networks platform
Copyright (C) 2009 SocialABC, Inc. http://www.socialabc.com
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program in a file called LICENSE; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/ 

class Event extends Entity
{
    public $EventID          = -1;
    public $OrganizationID   = -1;
    public $UserID           = -1;
    public $CommunityID      = -1;
    public $GroupID          = -1;

    public $EventName        				= '';
    public $EventDescription                = '';
    public $EventType		 				= EVENT_TYPE_PERSONAL;
    public $EventSubType	 				= '';
    public $EventAvailability               = EVENT_AVAILABILITY_PUBLIC;
    public $EventCost                   	= '';
    public $EventPicture1                  	= '';
    public $EventPicture2                  	= '';
    public $EventPicture3                  	= '';
    public $EventPicture4                  	= '';

    public $EventStartDate                  = '';
    public $EventEndDate                    = '';
    public $EventReminder                   = EVENT_REMINDER_NONE;
    public $EventReminderType               = EVENT_REMINDERTYPE_NONE;

    public $EventLocationName           	= '';
    public $EventLocationStreet        		= '';
    public $EventLocationCountry        	= '';
    public $EventLocationState          	= '';
    public $EventLocationCity           	= '';
    public $EventLocationZip            	= '';

    public $EventContactPhone           	= '';
    public $EventContactEmail           	= '';


    public function isSubscriptionRequested($userID)
    {
    	return Message::isEventRequestSent($userID, $this->EventID);
    }

    public function getOwner()
    {
    	if ($this->OrganizationID != -1)
    	{
    		return new Organization($this->OrganizationID);
    	}
    	else if ($this->CommunityID != -1)
    	{
    		return new Community($this->CommunityID);
    	}
    	else if ($this->GroupID != -1)
    	{
    		return new Group($this->GroupID);
    	}
    	else
    	{
    		return new User($this->UserID);
		}
    }

    public function addParticipant($userID)
    {
    	if (!$this->isSubscribed($userID))
    	{
    		$db = DBConnect::getInstance();

    		$db->insert
    		(
    			'eventtoparticipant',

    			array
    			(
    				'UserID'	=> $userID,
    				'EventID'	=> $this->EventID,
    				'EventToParticipantStatus' => USER_EVENT_MEMBER
    			)
    		);
    	}
    }

    public function removeAdmin($userID)
    {
        if ($this->isSubscribed($userID))
        {
            $db = DBConnect::getInstance();

            $db->update
            (
                'eventtoparticipant',

                array
                (
                    'EventToParticipantStatus' => USER_EVENT_MEMBER
                ),

                'UserID = %d and EventID = %d',
                $userID, $this->EventID
            );
        }
    }

    public function makeAdmin($userID)
    {
        if ($this->isSubscribed($userID))
        {
            $db = DBConnect::getInstance();

            $db->update
            (
                'eventtoparticipant',

                array
                (
                    'EventToParticipantStatus' => USER_EVENT_ADMIN
                ),

                'UserID = %d and EventID = %d',
                $userID, $this->EventID
            );
        }
    }

    public function unsubscribe($userID)
    {
    	$db = DBConnect::getInstance();
    	$db->delete('eventtoparticipant', 'UserID = %d', $userID);
    }

    public function isOwner($userID)
    {
    	return ($this->UserID == $userID) || User::checkEmployer($userID);
    }

    public function isAdmin($userID)
    {
    	$db = DBConnect::getInstance();

    	$result = $db->query('select count(*) as cnt from `eventtoparticipant` ' .
    		'where EventToParticipantStatus = %d AND UserID = %d and EventID = %d',
    		USER_EVENT_ADMIN, $userID, $this->EventID);

    	return ($result->Data[0]['cnt'] > 0) || $this->isOwner($userID) || User::checkEmployer($userID);
    }

    public function isSubscribed($userID)
    {
    	$db = DBConnect::getInstance();

    	$result = $db->query('select count(*) as cnt from `eventtoparticipant` where UserID = %d and EventID = %d',
    		$userID, $this->EventID);

    	return ($result->Data[0]['cnt'] > 0) || User::checkEmployer($userID);
    }

    public static function searchEvent($ownerEntity, $ownerID, $eventName, $eventStartDate)
    {
    	$db = DBConnect::getInstance();

    	$return = new Event();
    	$return->{$ownerEntity . 'ID'} = $ownerID;
    	$return->EventName = $eventName;
    	$return->EventStartDate = $eventStartDate;

    	if ($ownerEntity == 'User')
        {
            $result = $db->query('select e.* from `event` e left join `eventtoparticipant` ep ' .
                'on e.EventID = ep.EventID where (e.UserID = %d or ep.UserID = %d) and EventName = %s ' .
                'and EventStartDate = %s group by e.EventID',
                $ownerID, $ownerID,
                $eventName, $eventStartDate);
        }
        else
        {
            $result = $db->query('select * from `event` where e.%f = %d and EventName = %s ' .
                'and EventStartDate = %s', $ownerEntity . 'ID', $ownerID, $eventName, $eventStartDate);
        }

    	if ($result->Size > 0)
    	{

    		$return->populateFromArray($result->Data[0]);
    	}

    	return $return;
    }

    public static function getUserOwnEvents($userID)
    {
    	$db = DBConnect::getInstance();
        $return = array();


        $result = $db->query('select e.* from `event` e left join `eventtoparticipant` ep on e.EventID = ep.EventID ' .
        	'where e.UserID = %d or ep.UserID = %d and e.EventType = %d order by EventStartDate',
        	$userID, $userID, EVENT_TYPE_GROUP);

        foreach ($result->Data as $row)
        {
            $event = new Event();
            $event->populateFromArray($row);
            $return[] = $event;
        }

        return $return;
    }

    public static function getUserUpcomingEvents($userID, $limit = 0)
    {
        $db = DBConnect::getInstance();
        $return = array();

        $result = $db->query('select e.* from `event` e ' .
            'left join `eventtoparticipant` ep on e.EventID = ep.EventID ' .
            'where (e.UserID = %d or ep.UserID = %d) and e.EventStartDate >= \'' .
            date('Y-m-d') . '\' group by e.EventID order by EventStartDate' .
            ($limit ? ' limit ' . intval($limit) : ''),
            $userID, $userID);

        foreach ($result->Data as $row)
        {
            $event = new Event();
            $event->populateFromArray($row);
            $return[] = $event;
        }

        return $return;
    }

    public static function getUserEvents($userID)
    {
        $db = DBConnect::getInstance();
        $return = array();

        $result = $db->query('SELECT e.* FROM `event` AS e LEFT JOIN `connection` AS cg ON cg.GroupID <> -1 AND cg.UserID = %d ' .
            'LEFT JOIN `connection` AS co ON co.OrganizationID <> -1 AND co.UserID = %d LEFT JOIN `eventtoparticipant` AS ep ON e.EventID = ep.EventID ' .
        	'WHERE (e.UserID = %d OR ep.UserID = %d OR co.OrganizationID = e.OrganizationID OR cg.GroupID = e.GroupID) GROUP BY e.EventID ORDER BY EventStartDate', $userID, $userID, $userID, $userID);

        foreach ($result->Data as $row)
        {
            $event = new Event();
            $event->populateFromArray($row);
            $return[] = $event;
        }

        return $return;
    }

    public static function getCommunityEvents($communityID)
    {
        $db = DBConnect::getInstance();
        $return = array();

        $result = $db->query('select * from `event` where CommunityID = %d order by EventStartDate', $communityID);

        foreach ($result->Data as $row)
        {
            $event = new Event();
            $event->populateFromArray($row);
            $return[] = $event;
        }

        return $return;
    }

    public static function getGroupEvents($groupID)
    {
        $db = DBConnect::getInstance();
        $return = array();

        $result = $db->query('select * from `event` where GroupID = %d order by EventStartDate', $groupID);

        foreach ($result->Data as $row)
        {
            $event = new Event();
            $event->populateFromArray($row);
            $return[] = $event;
        }

        return $return;
    }

    public static function getOrganizationEvents($orgID)
    {
        $db = DBConnect::getInstance();
        $return = array();

        $result = $db->query('select * from `event` where OrganizationID = %d order by EventStartDate', $orgID);

        foreach ($result->Data as $row)
        {
            $event = new Event();
            $event->populateFromArray($row);
            $return[] = $event;
        }

        return $return;
    }
    
    public static function getPublicEvents()
    {
        $db = DBConnect::getInstance();
        
        $admins = array();
        $return = array();

        $result = $db->query('select * from `user` where `UserType` = %d', USER_TYPE_EMPLOYER);
        foreach ($result->Data as $admin)
        {
        	$admins[]=$admin["UserID"];
        }
        //print_r($admins);

        
        $result = $db->query('select * from `event` where UserID IN('.implode(", ", $admins).') AND GroupID=-1  AND OrganizationID=-1 AND CommunityID=-1 AND EventAvailability = %d order by EventStartDate', EVENT_AVAILABILITY_PUBLIC);
        //print_r($result);
        foreach ($result->Data as $row)
        {
            $event = new Event();
            $event->populateFromArray($row);
            $return[] = $event;
        }
        return $return;
    }

}
?>