<?php
/*
NetworX - open-source social networks platform
Copyright (C) 2009 SocialABC, Inc. http://www.socialabc.com
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program in a file called LICENSE; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/ 

class Message extends Entity
{
    public $MessageID       = -1;
    public $OrganizationID  = -1;
    public $CommunityID     = -1;
    public $ObjectID        = -1;
    public $UserID          = -1;
    public $GroupID         = -1;
    public $use_UserID      = -1;
    public $MessageSubject  = "";
    public $MessageText     = "";
    public $MessageHtml     = "";
    public $MessageDate     = "";
    public $MessageStatus   = -1;
    public $MessageType     = -1;
    public $MessageFlags     = 0;
    public $MessageRecipient = "";
    public $MessageFile     = "";


	public function saveToDB($decode = false)
	{
        if ($decode)
        {
        	$this->MessageText = html_entity_decode($this->MessageText);
        }
		parent::saveToDB();
	}

    public function populateFromArray($data, $decode = true)
    {
        parent::populateFromArray($data);
        if ($decode)
        {
        	$this->MessageText = html_entity_decode($this->MessageText);
        }
    }

	public function setReadStatus()
	{
		$this->MessageStatus = MESSAGE_STATUS_READ;
		if ($this->MessageID)
		{
			$this->saveToDB();
		}
	}

	public static function sendEmail($strTo, $strSubject, $strBody)
    {
    	mail($strTo, $strSubject, $strBody, 'From: ' . SYSTEM_EMAIL . "\r\n" . 'Content-type: text/html');
    }

    public static function getMessageTemplate($templateFile)
    {
		$tpl = new HTML_Template_Sigma(PATH_MESSAGES, PATH_MESSAGES_CACHE);
		$tpl->loadTemplateFile($templateFile);
		tpl_set_common_functions($tpl);
		$tpl->setGlobalVariable('SITE_URL', SITE_URL);
		return $tpl;
    }

    public static function sendAllMessage($userID, $messageSubject, $messageText)
    {
        $messageDate = date("Y-m-d H:i:s");

        $db = DBConnect::getInstance();
        $db->query('INSERT INTO %t (UserID, use_UserID, MessageRecipient, MessageSubject, MessageText, MessageDate, MessageStatus, MessageType) SELECT %d, UserID, concat_ws(\' \', UserFirstname, UserLastname), %s, %s, %s, %d, %d FROM `user`', 'message', $userID, $messageSubject, $messageText, $messageDate, MESSAGE_STATUS_UNREAD, MESSAGE_OFFICIAL);
    }


    public static function sendGroupEmail($groupID, $messageSubject, $messageText)
    {
        $db = DBConnect::getInstance();
    	$result = $db->query('SELECT u.UserEmail FROM `connection` c inner join user u on c.UserID = u.UserID where c.GroupID = %d', $groupID);

    	foreach ($result->Data as $row)
    	{
    		Message::sendEmail($row['UserEmail'], $messageSubject, $messageText);
    	}
    }

    public static function sendOrganizationEmail($orgID, $messageSubject, $messageText)
    {
        $db = DBConnect::getInstance();
    	$result = $db->query('SELECT u.UserEmail FROM `connection` c inner join user u on c.UserID = u.UserID where c.OrganizationID = %d', $orgID);

    	foreach ($result->Data as $row)
    	{
    		Message::sendEmail($row['UserEmail'], $messageSubject, $messageText);
    	}
    }

    public static function sendUserEmail($userID, $messageSubject, $messageText)
    {
        $db = DBConnect::getInstance();
    	$result = $db->query('SELECT UserEmail FROM `user` where UserID = %d', $userID);

    	if ($result->Data > 0)
    	{
    		Message::sendEmail($result->Data[0]['UserEmail'], $messageSubject, $messageText);
    	}
    }

    public static function sendAllEmail($messageSubject, $messageText)
    {
        $db = DBConnect::getInstance();
    	$result = $db->query('SELECT UserEmail FROM `user`', $userID);

    	foreach ($result->Data as $row)
    	{
    		Message::sendEmail($row['UserEmail'], $messageSubject, $messageText);
    	}
    }

    public static function sendUserMessage($userID, $use_UserID, $messageSubject, $messageText, $filename = '')
    {
		$user = new User($userID);
    	$recipient = new User($use_UserID);
    	if ($user->UserID != -1 && $recipient->UserID != -1)
    	{
	        $connections = $recipient->getConnections(array('ConnectionType' => CONNECTIONTYPE_PREFERENCES, 'ToUserID' => $user->UserID));
	        if (count($connections) > 0)
	        {
	            $connection = $connections[0];
	        }
	        else
	        {
	            $connection = new Connection();
	        }

	        if (!($connection->Flags & USER_PREFERENCES_MESSAGES))
	        {
    			$message = new Message();
		        $message->UserID = $userID;
		        $message->use_UserID = $use_UserID;
		        $message->MessageRecipient = $recipient->UserFirstname . ' ' . $recipient->UserLastname;
		        $message->MessageSubject = $messageSubject;
		        $message->MessageText = $messageText;
		        $message->MessageType = MESSAGE_MEMBER_MEMBER;
		        $message->MessageStatus = MESSAGE_STATUS_UNREAD;
		        $message->MessageDate = date("Y-m-d H:i:s");
		        $message->MessageFile = $filename;
		        $message->saveToDB();

		        $message->MessageID = -1;
		        $message->MessageStatus = MESSAGE_STATUS_SENT;
		        $message->saveToDB();

				$session = Session::getUserSession($use_UserID);

		        if ($session->SessionID != -1 && $session->UserOnlineStatus == USER_ONLINE_STATUS_AWAY)
		        {
    				$message = new Message();
			        $message->UserID = $use_UserID;
			        $message->use_UserID = $userID;
			        $message->MessageSubject = "I'm currently away";
			        $message->MessageText = $session->UserAwayMessage;
			        $message->MessageType = MESSAGE_MEMBER_MEMBER;
			        $message->MessageStatus = MESSAGE_STATUS_UNREAD;
			        $message->MessageDate = date("Y-m-d H:i:s");
			        $message->saveToDB();
				}
			}
		}
    }

    public static function sendGroupMessage($userID, $groupID, $messageSubject, $messageText, $filename)
    {
        $group = new Group($groupID);
        if ($group->GroupID != -1)
        {
            $owner = $group->getOwner();
            if (get_class($owner) == 'Organization')
            {
                $owner = $owner->getOwner();
            }

            if ($owner->UserID == $userID)
            {
                return Message::sendGroupUsersMessage($userID, $groupID, $messageSubject, $messageText, $filename);
            }
            else
            {
                $message = new Message();
                $message->UserID = $userID;
                $message->use_UserID = $owner->UserID;
                $message->MessageRecipient = $group->GroupName;
                $message->MessageSubject = $messageSubject;
                $message->MessageText = $messageText;
                $message->MessageType = MESSAGE_MEMBER_GROUP;
                $message->ObjectID = $group->GroupID;
                $message->MessageStatus = MESSAGE_STATUS_UNREAD;
                $message->MessageDate = date("Y-m-d H:i:s");
                $message->MessageFile = $filename;
                $message->saveToDB();
            }
        }
    }

    public static function sendGroupUsersMessage($userID, $groupID, $messageSubject, $messageText, $filename)
    {
    	$group = new Group($groupID);
    	if ($group->GroupID != -1)
    	{
	        $messageDate = date("Y-m-d H:i:s");

	        $db = DBConnect::getInstance();
	        $db->query('INSERT INTO %t (UserID, use_UserID, GroupID, MessageRecipient, MessageSubject, MessageText, MessageDate, MessageStatus, MessageType, ObjectID, MessageFile) SELECT %d, c.UserID, c.GroupId, %s, %s, %s, %s, %d, %d, %d, %s FROM `connection` c where c.GroupID = %d and ConnectionType != %d', 'message', $userID, $group->GroupName . ' Group', $messageSubject, $messageText, $messageDate, MESSAGE_STATUS_UNREAD, MESSAGE_MEMBER_GROUP, $groupID, $filename, $groupID, CONNECTIONTYPE_OWNER);

            $message = new Message();
            $message->UserID = $userID;
            $message->MessageRecipient = $group->GroupName;
            $message->MessageSubject = $messageSubject;
            $message->MessageText = $messageText;
            $message->MessageType = MESSAGE_MEMBER_GROUP;
            $message->MessageStatus = MESSAGE_STATUS_SENT;
            $message->MessageFile = $filename;
            $message->MessageDate = date("Y-m-d H:i:s");
            $message->saveToDB();
		}
    }

    public static function sendOrganizationMessage($userID, $organizationID, $messageSubject, $messageText, $filename)
    {
        $organization = new Organization($organizationID);
        if ($organization->OrganizationID != -1)
        {
            $owner = $organization->getOwner();

            if ($owner->UserID == $userID)
            {
                return Message::sendOrganizationUsersMessage($userID, $organizationID, $messageSubject, $messageText, $filename);
            }
            else
            {
                $message = new Message();
                $message->UserID = $userID;
                $message->use_UserID = $owner->UserID;
                $message->MessageRecipient = $organization->OrganizationName;
                $message->MessageSubject = $messageSubject;
                $message->MessageText = $messageText;
                $message->MessageType = MESSAGE_MEMBER_ORGANIZATION;
                $message->ObjectID = $organization->OrganizationID;
                $message->MessageStatus = MESSAGE_STATUS_UNREAD;
                $message->MessageDate = date("Y-m-d H:i:s");
                $message->MessageFile = $filename;
                $message->saveToDB();
            }
        }
    }

    public static function sendOrganizationUsersMessage($userID, $organizationID, $messageSubject, $messageText, $filename)
    {
    	$organization = new Organization($organizationID);
    	if ($organization->OrganizationID != -1)
    	{
	        $messageDate = date("Y-m-d H:i:s");
	        $db = DBConnect::getInstance();
	        $db->query('INSERT INTO %t (UserID, use_UserID, OrganizationID, MessageRecipient, MessageSubject, MessageText, MessageDate, MessageStatus, MessageType, ObjectID, MessageFile) SELECT %d, c.UserID, c.OrganizationID, %s, %s, %s, %s, %d, %d, %d, %s FROM `connection` c where c.OrganizationID = %d and c.ConnectionType != %d', 'message', $userID, $organization->OrganizationName . ' Organization', $messageSubject, $messageText, $messageDate, MESSAGE_STATUS_UNREAD, MESSAGE_MEMBER_ORGANIZATION, $organizationID, $filename, $organizationID, CONNECTIONTYPE_OWNER);

            $message = new Message();
            $message->UserID = $userID;
            $message->MessageRecipient = $organization->OrganizationName;
            $message->MessageSubject = $messageSubject;
            $message->MessageText = $messageText;
            $message->MessageType = MESSAGE_MEMBER_ORGANIZATION;
            $message->MessageStatus = MESSAGE_STATUS_SENT;
            $message->MessageDate = date("Y-m-d H:i:s");
            $message->MessageFile = $filename;
            $message->saveToDB();
		}
    }

    public static function sendCommunityMessage($userID, $communityID, $messageSubject, $messageText, $filename)
    {
        $community = new Community($communityID);
        if ($community->CommunityID != -1)
        {
            $owner = $community->getOwner();
            if ($owner->UserID == $userID)
            {
                return Message::sendCommunityUsersMessage($userID, $communityID, $messageSubject, $messageText, $filename);
            }
            else
            {
                $message = new Message();
                $message->UserID = $userID;
                $message->use_UserID = $owner->UserID;
                $message->MessageRecipient = $community->CommunityName;
                $message->MessageSubject = $messageSubject;
                $message->MessageText = $messageText;
                $message->MessageType = MESSAGE_MEMBER_COMMUNITY;
                $message->ObjectID = $community->CommunityID;
                $message->MessageStatus = MESSAGE_STATUS_UNREAD;
                $message->MessageDate = date("Y-m-d H:i:s");
                $message->MessageFile = $filename;
                $message->saveToDB();
            }
        }
    }

    public static function sendCommunityUsersMessage($userID, $communityID, $messageSubject, $messageText, $filename)
    {
    	$community = new Community($communityID);
    	if ($community->CommunityID != -1)
    	{
	        $messageDate = date("Y-m-d H:i:s");
	        $db = DBConnect::getInstance();
	        $db->query('INSERT INTO %t (UserID, use_UserID, CommunityID, MessageRecipient, MessageSubject, MessageText, MessageDate, MessageStatus, MessageType, ObjectID, MessageFile) SELECT %d, c.UserID, c.CommunityID, %s, %s, %s, %s, %d, %d, %d, %s FROM `connection` c where c.CommunityID = %d and c.ConnectionType != %d', 'message', $userID, $community->CommunityName . ' Community', $messageSubject, $messageText, $messageDate, MESSAGE_STATUS_UNREAD, MESSAGE_MEMBER_COMMUNITY, $communityID, $filename, $communityID, CONNECTIONTYPE_OWNER);

            $message = new Message();
            $message->UserID = $userID;
            $message->MessageRecipient = $community->CommunityName;
            $message->MessageSubject = $messageSubject;
            $message->MessageText = $messageText;
            $message->MessageType = MESSAGE_MEMBER_COMMUNITY;
            $message->MessageStatus = MESSAGE_STATUS_SENT;
            $message->MessageDate = date("Y-m-d H:i:s");
            $message->MessageFile = $filename;
            $message->saveToDB();
		}
    }


    public static function isEventRequestSent($userID, $eventID)
    {
    	$db = DBConnect::getInstance();
    	$result = $db->query('select count(*) as cnt from message where ObjectID = %d and MessageType = %d and UserID = %d',
    		$eventID, MESSAGE_EVENT_REQUEST, $userID);

		return $result->Data[0]['cnt'] > 0;
    }

	public static function sendEventRequest($userID, $eventID, $messageText)
    {
    	$event = new Event($eventID);
    	$user  = new User($userID);
    	$recipient = new User($event->UserID);

    	if ($recipient->UserID != -1 && $user->UserID != -1)
    	{
    		$message = new Message();
    		$message->ObjectID = $eventID;
		    $message->UserID = $user->UserID;
		    $message->use_UserID = $recipient->UserID;
		    $message->MessageRecipient = $recipient->UserFirstname . ' ' . $recipient->UserLastname;
		    $message->MessageSubject = $user->UserFirstname . ' ' . $user->UserLastname . ' requested subscription to ' . $event->EventName;
		    $message->MessageText = $messageText;
		    $message->MessageType = MESSAGE_EVENT_REQUEST;
		    $message->MessageStatus = MESSAGE_STATUS_UNREAD;
		    $message->MessageDate = date("Y-m-d H:i:s");
		    $message->saveToDB();
		}
    }

    public static function isGroupRequestSent($id, $groupID, $entity = 'User')
    {
		$db = DBConnect::getInstance();
    		$result = $db->query('select count(*) as cnt from message where ObjectID = %d and MessageType = %d and ' . $entity . 'ID = %d',
    			$groupID, MESSAGE_GROUP_REQUEST, $id);
		return $result->Data[0]['cnt'] > 0;
    }

    public static function isEventInviteSent($userID, $eventID)
    {
		$db = DBConnect::getInstance();
    		$result = $db->query('select count(*) as cnt from message where ObjectID = %d and MessageType = %d and UserID = %d',
    			$eventID, MESSAGE_EVENT_INVITE, $userID);
		return $result->Data[0]['cnt'] > 0;
    }

    public static function isBuddyRequestSent($userID1, $userID2)
    {
		$db = DBConnect::getInstance();
   		$result1 = $db->query('select count(*) as cnt from message where ObjectID = %d ' .
    			'and MessageType = %d and use_UserID = %d',
    			$userID1, MESSAGE_BUDDY_REQUEST, $userID2);

   		$result2 = $db->query('select count(*) as cnt from message where ObjectID = %d ' .
    			'and MessageType = %d and use_UserID = %d',
    			$userID2, MESSAGE_BUDDY_REQUEST, $userID1);

		return ($result1->Data[0]['cnt'] > 0 || $result2->Data[0]['cnt'] > 0);
    }

	public static function sendBuddyRequest($userID, $toUserID, $messageText)
    {
    	$user	= new User($userID);
    	$toUser	= new User($toUserID);

    	if ($user->UserID != -1 && $toUser->UserID != -1)
    	{
    		$message = new Message();
    		$message->ObjectID = $userID;
		    $message->UserID = $user->UserID;
		    $message->use_UserID = $toUserID;
		    $message->MessageRecipient = $toUser->UserFirstname . ' ' . $toUser->UserLastname;
		    $message->MessageSubject = $user->UserFirstname . ' ' . $user->UserLastname . ' requested a friendship';
		    $message->MessageText = $messageText;
		    $message->MessageType = MESSAGE_BUDDY_REQUEST;
		    $message->MessageStatus = MESSAGE_STATUS_UNREAD;
		    $message->MessageDate = date("Y-m-d H:i:s");
		    $message->saveToDB();
		}
    }

	public static function sendGroupRequest($entity, $id, $groupID, $messageText = '', $type = 0)
    {
    	global $user;

    	$group = new Group($groupID);
    	$obj   = Entity::getObject($entity, $id);
    	$owner = $group->getOwner();

    	if ($owner->UserID != -1 && $obj->{db_get_entity_key($obj)} != -1)
    	{
    		$message = new Message();
    		$message->ObjectID = $groupID;
		    $message->{$entity . 'ID'} = $id;
		    $message->UserID = $user->UserID;
		    $message->use_UserID = $owner->UserID;
		    $message->MessageRecipient = $owner->UserFirstname . ' ' . $owner->UserLastname;

		    if ($type)
		    {
		    	$message->MessageFlags = $type;
				$message->MessageSubject = db_get_entity_display_name($obj) .
						' requested ' . ($type == CONNECTIONFLAG_TYPE_OFFICIAL ? 'official' : 'unofficial') .
						' connection to ' . $group->GroupName;
			}
			else
			{
				$message->MessageSubject = db_get_entity_display_name($obj) .
					' requested subscription to ' . $group->GroupName;
			}
			$message->MessageText = $messageText;

		    $message->MessageText = $messageText;
		    $message->MessageType = MESSAGE_GROUP_REQUEST;
		    $message->MessageStatus = MESSAGE_STATUS_UNREAD;
		    $message->MessageDate = date("Y-m-d H:i:s");
		    $message->saveToDB();
		}
    }

    public static function isOrganizationRequestSent($id, $orgID, $entity = 'User')
    {
		$db = DBConnect::getInstance();
    		$result = $db->query('select count(*) as cnt from message where ObjectID = %d and MessageType = %d and ' . $entity . 'ID = %d',
    			$orgID, MESSAGE_ORGANIZATION_REQUEST, $id);
		return $result->Data[0]['cnt'] > 0;
    }


	public static function sendOrganizationRequest($entity, $id, $orgID, $messageText = '', $type = 0)
    {
    	global $user;

    	$org   = new Organization($orgID);
    	$obj   = Entity::getObject($entity, $id);
    	$owner = $org->getOwner();

    	if ($owner->UserID != -1 && $obj->{db_get_entity_key($obj)} != -1)
    	{
    		$message = new Message();
    		$message->{$entity . 'ID'} = $id;
    		$message->ObjectID = $org->OrganizationID;
		    $message->UserID = $user->UserID;
		    $message->use_UserID = $owner->UserID;
		    $message->MessageRecipient = $owner->UserFirstname . ' ' . $owner->UserLastname;
		    if ($type)
		    {
		    	$message->MessageFlags = $type;
			    $message->MessageSubject = db_get_entity_display_name($obj) . ' requested ' .
			    	($type == CONNECTIONFLAG_TYPE_OFFICIAL ? 'official' : 'unofficial') .
			    	' connection to ' . $org->OrganizationName;
			}
			else
			{
		    	$message->MessageSubject = db_get_entity_display_name($obj) .
		    		' requested subscription to ' . $org->OrganizationName;
			}
		    $message->MessageText = $messageText;
		    $message->MessageType = MESSAGE_ORGANIZATION_REQUEST;
		    $message->MessageStatus = MESSAGE_STATUS_UNREAD;
		    $message->MessageDate = date("Y-m-d H:i:s");
		    $message->saveToDB();
		}
    }

	public static function sendUserTaggingRequest($from_user_id, $to_user_id, $tag_type, $object_id, $tag_id)
	{
		$message = new Message();
		$user  = new User($from_user_id);

		$message->ObjectID = $tag_id;
		$message->UserID = $from_user_id;
		$message->use_UserID = $to_user_id;

		switch ($tag_type)
		{
			case TAG_TYPE_BLOG_ENTRY:
				$message->MessageSubject = "Tagging Request";
				$message->MessageText = db_get_entity_display_name($user) . ' requested tagging this <a href="blogentry.php?id=' . $object_id . '">blog entry</a>. ';
			break;
		}
		$to_user = new User($to_user_id);
		$message->MessageRecipient = db_get_entity_display_name($to_user);

		$message->MessageType = MESSAGE_USER_TAGGING_REQUEST;
		$message->MessageStatus = MESSAGE_STATUS_UNREAD;
		$message->MessageDate = date("Y-m-d H:i:s");
		$message->saveToDB();
	}

	public static function sendGroupTaggingRequest($user_id, $group_id, $tag_type, $object_id, $tag_id)
	{
		$group = new Group($group_id);
		$message = new Message();
		$user  = new User($user_id);

		$message->ObjectID = $tag_id;
		$message->UserID = $user_id;
		$message->use_UserID = $group->getOwnerUserID();

		switch ($tag_type)
		{
			case TAG_TYPE_BLOG_ENTRY:
				$message->MessageSubject = "Tagging Request";
				$message->MessageText = db_get_entity_display_name($user) . ' requested tagging this <a href="blogentry.php?id=' . $object_id . '">blog entry</a>. ';
			break;
		}
		$to_user = new User($message->use_UserID);
		$message->MessageRecipient = db_get_entity_display_name($to_user);

		$message->MessageType = MESSAGE_GROUP_TAGGING_REQUEST;
		$message->MessageStatus = MESSAGE_STATUS_UNREAD;
		$message->MessageDate = date("Y-m-d H:i:s");
		$message->saveToDB();
	}

	public static function sendOrganizationTaggingRequest($user_id, $organization_id, $tag_type, $object_id, $tag_id)
	{
		$organization = new Organization($organization_id);
		$message = new Message();
		$user  = new User($user_id);

		$message->ObjectID = $tag_id;
		$message->UserID = $user_id;
		$message->use_UserID = $organization->getOwnerUserID();

		switch ($tag_type)
		{
			case TAG_TYPE_BLOG_ENTRY:
				$message->MessageSubject = "Tagging Request";
				$message->MessageText = db_get_entity_display_name($user) . ' requested tagging this <a href="blogentry.php?id=' . $object_id . '">blog entry</a>. ';
			break;
		}
		$to_user = new User($message->use_UserID);
		$message->MessageRecipient = db_get_entity_display_name($to_user);

		$message->MessageType = MESSAGE_ORGANIZATION_TAGGING_REQUEST;
		$message->MessageStatus = MESSAGE_STATUS_UNREAD;
		$message->MessageDate = date("Y-m-d H:i:s");
		$message->saveToDB();
	}

    public static function sendGroupInvite($fromUserID, $groupID, $userID, $messageText = '')
    {
    	$group = new Group($groupID);
    	$user = new User($userID);

    	if ($user->UserID != -1 && $group->GroupID != -1 && $group->userIsAdmin($fromUserID))
    	{
    		$message = new Message();
    		$message->ObjectID = $groupID;
	        $message->UserID = $fromUserID;
	        $message->use_UserID = $user->UserID;
	        $message->MessageRecipient = $user->UserFirstname . ' ' . $user->UserLastname;
	        $message->MessageSubject = 'You have been invited to ' . $group->GroupName;
	        $message->MessageText = $messageText;
	        $message->MessageType = MESSAGE_GROUP_INVITE;
	        $message->MessageStatus = MESSAGE_STATUS_UNREAD;
	        $message->MessageDate = date("Y-m-d H:i:s");
	        $message->saveToDB();
		}
    }

    public static function sendGroupAdminInvite($fromUserID, $userID, $groupID, $messageText = '')
    {
    	$group = new Group($groupID);
    	$user  = new User($userID);
    	if ($group->GroupID != -1 && $user->UserID != -1 && $group->userIsAdmin($fromUserID))
    	{
    		$message = new Message();
    		$message->ObjectID = $group->GroupID;
	        $message->UserID = $fromUserID;
	        $message->use_UserID = $user->UserID;
	        $message->MessageRecipient = $user->UserFirstname . ' ' . $user->UserLastname;
	        $message->MessageSubject = 'You have been invited to be admin on ' . $group->GroupName;
	        $message->MessageText = $messageText;
	        $message->MessageType = MESSAGE_GROUP_ADMIN_INVITE;
	        $message->MessageStatus = MESSAGE_STATUS_UNREAD;
	        $message->MessageDate = date("Y-m-d H:i:s");
	        $message->saveToDB();
    	}
    }

    public static function sendOrganizationInvite($fromUserID, $orgID, $userID, $messageText)
    {
    	$org = new Organization($orgID);
    	$user = new User($userID);
    	if ($user->UserID != -1 && $org->OrganizationID != -1 && $org->userIsAdmin($fromUserID))
    	{
    		$message = new Message();
    		$message->ObjectID = $org->OrganizationID;
	        $message->UserID = $fromUserID;
	        $message->use_UserID = $user->UserID;
	        $message->MessageRecipient = $user->UserFirstname . ' ' . $user->UserLastname;
	        $message->MessageSubject = 'You have been invited to ' . $org->OrganizationName;
	        $message->MessageText = $messageText;
	        $message->MessageType = MESSAGE_ORGANIZATION_INVITE;
	        $message->MessageStatus = MESSAGE_STATUS_UNREAD;
	        $message->MessageDate = date("Y-m-d H:i:s");
	        $message->saveToDB();
		}
    }

	public static function sendOrganizationAdminInvite($fromUserID, $userID, $orgID, $messageText)
    {
    	$org = new Organization($orgID);
    	$user  = new User($userID);
    	if ($org->OrganizationID != -1 && $user->UserID != -1 && $org->userIsAdmin($fromUserID))
    	{
    		$message = new Message();
    		$message->ObjectID = $org->OrganizationID;
	        $message->UserID = $fromUserID;
	        $message->use_UserID = $user->UserID;
	        $message->MessageRecipient = $user->UserFirstname . ' ' . $user->UserLastname;
	        $message->MessageSubject = 'You have been invited to be admin on ' . $org->OrganizationName;
	        $message->MessageText = $messageText;
	        $message->MessageType = MESSAGE_ORGANIZATION_ADMIN_INVITE;
	        $message->MessageStatus = MESSAGE_STATUS_UNREAD;
	        $message->MessageDate = date("Y-m-d H:i:s");
	        $message->saveToDB();
    	}
    }

    public static function sendEntityInvite($entity, $id, $fromUserID, $userID, $messageText)
    {
        switch ($entity)
        {
            case 'organization':
                return Message::sendOrganizationInvite($fromUserID, $id, $userID, $messageText);
            case 'group':
                return Message::sendGroupAdminInvite($fromUserID, $id, $userID, $messageText);
        }
    }

    public static function sendEventInvite($userID, $use_UserID, $eventID, $messageText)
    {
    	$event = new Event($eventID);
    	$user = new User($use_UserID);

    	if ($user->UserID != -1 && $event->EventID != -1)
    	{
    		$message = new Message();
    		$message->ObjectID = $eventID;
	        $message->UserID = $userID;
	        $message->use_UserID = $user->UserID;
	        $message->MessageRecipient = $user->UserFirstname . ' ' . $user->UserLastname;
	        $message->MessageSubject = 'You have been invited to ' . $event->EventName;
	        $message->MessageText = $messageText;
	        $message->MessageType = MESSAGE_EVENT_INVITE;
	        $message->MessageStatus = MESSAGE_STATUS_UNREAD;
	        $message->MessageDate = date("Y-m-d H:i:s");
	        $message->saveToDB();
		}
    }

    public static function getUnreadUserMessagesCount($userID)
    {
        $db = DBConnect::getInstance();
        $res = $db->query('select count(*) as c from %t where use_UserID = %d and MessageStatus = ' . MESSAGE_STATUS_UNREAD, 'message', $userID);
        return $res->Data[0]['c'];
    }

    public static function getUnreadUserMessagesCountForType($userID, $type)
    {
        $db = DBConnect::getInstance();
        $res = $db->query('select count(*) as c from %t where use_UserID = %d and MessageType = %d AND MessageStatus = ' . MESSAGE_STATUS_UNREAD, 'message', $userID, $type);
        return $res->Data[0]['c'];
    }

    public static function getSentMessages($userID, $messageType = -1)
    {
        $db = DBConnect::getInstance();
        $return = array();
        if (isset($messageType) && $messageType != -1)
        {
            $where = 'and MessageType = %d';
        } else
        {
            $messageType = '';
        }
        $result = $db->query('select * from %t where UserID = %d and MessageStatus = %d '.$where, 'message', $userID, MESSAGE_STATUS_SENT ,$messageType);

        foreach ($result->Data as $row)
        {
            $message = new Message();
            $message->populateFromArray($row);
            $return[] = $message;
        }
        return $return;
    }

    public static function getInboxMessages($userID, $messageType = -1)
    {
        $db = DBConnect::getInstance();
        $return = array();
        if (isset($messageType) && $messageType != -1)
        {
            $where = 'and MessageType = %d';
        } else
        {
            $messageType = '';
        }

        $result = $db->query('select * from %t where use_UserID = %d and (MessageStatus = %d or MessageStatus = %d) '.$where, 'message', $userID, MESSAGE_STATUS_UNREAD, MESSAGE_STATUS_READ ,$messageType);

        foreach ($result->Data as $row)
        {
            $message = new Message();
            $message->populateFromArray($row);
            $return[] = $message;
        }
        return $return;
    }
    public static function getTrashMessages($userID, $messageType = -1)
    {
        $db = DBConnect::getInstance();
        $return = array();
        $where='';
        if (isset($messageType) && $messageType != -1)
        {
            $where = 'and MessageType = %d';
        }
        $result = $db->query('select * from %t where ((UserID = %d and MessageStatus = %d) or (use_UserID = %d and MessageStatus = %d)) '.$where, 'message', $userID, MESSAGE_STATUS_SENT_DELETED, $userID, MESSAGE_STATUS_DELETED ,$messageType);

        foreach ($result->Data as $row)
        {
            $message = new Message();
            $message->populateFromArray($row);
            $return[] = $message;
        }
        return $return;
    }
    public static function deleteMessages($userID, $messageType = -1, $messageStatus = -1)
    {
        $db = DBConnect::getInstance();
        $where='';
        if (isset($messageType) && $messageType != -1)
        {
            $where.= ' and MessageType = %d';
            $arg1 = $messageType;
        }
        if (isset($messageStatus) && $messageStatus != -1)
        {
            $where.= ' and MessageStatus = %d';
            if (isset($arg1))
            {
                $arg2 = $messageStatus;
            } else
            {
                $arg1 = $messageStatus;
            }
        }
        $result = $db->query('DELETE FROM %t where UserID = %d '.$where, 'message', $userID, $arg1, $arg2);
    }

    public static function getDeletedMessagesCount($userID)
    {
        $db = DBConnect::getInstance();
        $res = $db->query('select count(*) as c from %t where use_UserID = %d and MessageStatus = %d or ' .
        	'UserID = %d and MessageStatus = %d', 'message', $userID, MESSAGE_STATUS_DELETED,
        	$userID, MESSAGE_STATUS_SENT_DELETED);

        return intval($res->Data[0]['c']);
    }

	public static function getGroupPendingUsers($group_id, $sort_by, $sort_order)
	{
		$return = array();

    	$db = DBConnect::getInstance();
    	$query = "SELECT DISTINCT u.* FROM `user` AS u
			INNER JOIN `message` AS m ON m.use_UserID = u.UserID
			WHERE (MessageType = %d OR MessageType = %d) AND ObjectID = %d
			UNION
			SELECT DISTINCT u.* FROM `user` AS u
			INNER JOIN `message` AS m ON m.UserID = u.UserID
			WHERE MessageType = %d AND ObjectID = %d
			ORDER BY %f %n";
    	$result = $db->query($query, MESSAGE_GROUP_INVITE, MESSAGE_GROUP_ADMIN_INVITE, $group_id, MESSAGE_GROUP_REQUEST, $group_id, $sort_by, $sort_order);

    	foreach ($result->Data as $row)
    	{
    		$group_user = new User();
    		$group_user->populateFromArray($row);
    		$return[] = $group_user;
    	}

    	return $return;
	}

	public static function getOrganizationPendingUsers($organization_id)
	{
		$return = array();

    	$db = DBConnect::getInstance();
    	$query = "SELECT DISTINCT u.* FROM `user` AS u
			INNER JOIN `message` AS m ON m.use_UserID = u.UserID
			WHERE (MessageType = %d OR MessageType = %d) AND ObjectID = %d
			UNION
			SELECT DISTINCT u.* FROM `user` AS u
			INNER JOIN `message` AS m ON m.UserID = u.UserID
			WHERE MessageType = %d AND ObjectID = %d
			";
    	$result = $db->query($query, MESSAGE_ORGANIZATION_INVITE, MESSAGE_ORGANIZATION_ADMIN_INVITE, $organization_id, MESSAGE_ORGANIZATION_REQUEST, $organization_id);

    	foreach ($result->Data as $row)
    	{
    		$organization_user = new User();
    		$organization_user->populateFromArray($row);
    		$return[] = $organization_user;
    	}

    	return $return;
	}

    public function moveToTrash()
    {
        if ($this->MessageStatus == MESSAGE_STATUS_SENT)
        {
            $this->MessageStatus = MESSAGE_STATUS_SENT_DELETED;
        }
        else
        {
            $this->MessageStatus = MESSAGE_STATUS_DELETED;
        }
        $this->saveToDB();
    }

    public function RestoreMessage()
    {
        if ($this->MessageStatus == MESSAGE_STATUS_DELETED)
        {
            $this->MessageStatus = MESSAGE_STATUS_READ;
        } elseif ($this->MessageStatus == MESSAGE_STATUS_SENT_DELETED)
        {
            $this->MessageStatus = MESSAGE_STATUS_SENT;
        }
        $this->saveToDB();
    }

    public static function getMessagesCount($userID, $messageType = -1, $messageStatus = -1)
    {
        $db = DBConnect::getInstance();
        if ($messageStatus == MESSAGE_STATUS_READ || $messageType == MESSAGE_STATUS_UNREAD)
        {
            $where = 'where use_UserID = %d';
        } else
        {
            $where = 'where UserID = %d';
        }
        if (isset($messageType) && $messageType != -1)
        {
            $where.= ' and MessageType = %d';
            $arg1 = $messageType;
        }
        if (isset($messageStatus) && $messageStatus != -1)
        {
            $where.= ' and MessageStatus = %d';
            if (isset($arg1))
            {
                $arg2 = $messageStatus;
            } else
            {
                $arg1 = $messageStatus;
            }
        }

        $res = $db->query('select count(*) as c from %t '.$where, 'message', $userID, $arg1, $arg2);
        return $res->Data[0]['c'];
    }

    public static function sendToEmail($messageSubject, $messageText, $email)
    {
        mail($email, $messageSubject, $messageText);
    }

    public static function emptyUserSentMessages($userID)
    {
    	$db = DBConnect::getInstance();
    	$query = $db->query('SELECT * FROM `message` WHERE UserID = %d and MessageStatus = %d', $userID, MESSAGE_STATUS_SENT);

    	foreach ($query->Data as $row)
    	{
    		$message = new Message();
    		$message->populateFromArray($row);
    		$message->moveToTrash();
    	}

    }

    public static function emptyUserTrash($userID)
    {
    	$db = DBConnect::getInstance();
    	$db->delete('message', '(UserID = %d AND MessageStatus = %d) OR (use_UserID = %d AND MessageStatus = %d)',
    		$userID, MESSAGE_STATUS_SENT_DELETED, $userID, MESSAGE_STATUS_DELETED);
    }
}
?>
