<?php
/*
NetworX - open-source social networks platform
Copyright (C) 2009 SocialABC, Inc. http://www.socialabc.com
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program in a file called LICENSE; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/ 

$section = 'messages';
require_once('includes/application_top.php');

if (!Session::isCurrentSessionLoggedIn())
{
	cmn_unauthorized_action();
}

$template->addBlockfile('MESSAGE_MENU', 'MESSAGE_MENU', 'message_menu.html');
$template->touchBlock('MESSAGE_MENU');

$events = $user->getEvents();
if (count($events))
{
	foreach ($events as $event)
	{
		if ($event->EventReminderType != EVENT_REMINDERTYPE_NONE)
		{
			$timestamp = time();
			$date_time_array = getdate($timestamp);

			$hours = $date_time_array['hours'];
			$minutes = $date_time_array['minutes'];
			$seconds = $date_time_array['seconds'];
			$month = $date_time_array['mon'];
			$day = $date_time_array['mday'];
			$year = $date_time_array['year'];

			switch ($event->EventReminder)
			{
				case EVENT_REMINDER_1DAY:	$day+=1; break;
				case EVENT_REMINDER_1HOUR:	$hours+=1; break;
				case EVENT_REMINDER_20MIN:	$minutes+=20; break;
				default: $hours+=1;
			}
			$remind_time = mktime($hours,$minutes,$seconds,$month,$day,$year);

			if (strtotime($event->EventStartDate) <= $remind_time)
			{
				$reminder_msg_tpl = Message::getMessageTemplate('event_reminder_message.html');
				$reminder_msg_tpl->setVariable('EventID', $event->EventID);
				$reminder_msg_tpl->setVariable('EventName', $event->EventName);
				$reminder_msg_tpl->setVariable('EventStartDate', date('l dS \of F Y h:i:s A', $event->EventStartDate));
				switch ($event->EventReminderType)
				{
					case EVENT_REMINDERTYPE_CELLECT:
						Message::sendUserMessage(1, $user->UserID, 'Event Reminder', $reminder_msg_tpl->get());
					break;
					case EVENT_REMINDERTYPE_EMAIL:
						Message::sendEmail($user->UserEmail, 'Event Reminder', $reminder_msg_tpl->get());
					break;
					default:
						Message::sendUserMessage(1, $user->UserID, 'Event Reminder', $reminder_msg_tpl->get());
						Message::sendEmail($user->UserEmail, 'Event Reminder', $reminder_msg_tpl->get());
				}
				$event->EventReminderType = EVENT_REMINDERTYPE_NONE;
				$event->saveToDb();
			}
		}
	}
}

if ( is_numeric($_GET['message_id']) || ($_POST['operation'] == 'view' && !empty($_POST['messages'])) || ($_POST['operation'] == 'reply' && !empty($_POST['messages'])) )
{
    if ($_POST['operation'] == 'delete')
    {
    	$messages = $_POST['messages'];
    	foreach ($messages as $message_id => $value)
		{
		    $message = new Message($message_id);

		    if ($message->UserID == $user->UserID || $message->use_UserID == $user->UserID)
		    {
		        $message->moveToTrash();
		    }
		}
		cmn_redirect('messages.php');
    }

	if (!empty($_POST['operation']))
	{
		$type = $_POST['operation'];
		$array_keys = array_keys($_POST['messages']);
		$message_id = $array_keys[0];
	}
	else
	{
		$message_id = $_GET['message_id'];
		$type = $_GET['type'];
	}

	$message = new Message($message_id);

	if ($message->MessageStatus == MESSAGE_STATUS_UNREAD)
	{
		if (($type == 'read') || ($type == 'view') || ($type == 'reply'))
		{
			$message->MessageStatus = MESSAGE_STATUS_READ;
	    	$message->saveToDB();
		}
	}

	$targetID = $user->UserID;
	switch ($message->MessageType)
	{
		case MESSAGE_BUDDY_REQUEST:  // ObjectID, UserID - requesting user
									 // use_UserID - invited user (current)
			$member = new User($message->ObjectID);

			switch (@$type)
			{
                case 'accept':
                    $member->addBuddy($targetID);
                    $message->MessageType = MESSAGE_BUDDY_ACCEPTED;
                    $message->saveToDB();
                    break;

                case 'decline':
                    $message->MessageType = MESSAGE_BUDDY_DECLINED;
                    $message->saveToDB();
                    break;

				default:
					$template->touchBlock('accept_decline');
			}
			break;

		case MESSAGE_EVENT_REQUEST:  // ObjectID - event id, UserID - requesting user
			$targetID = $message->UserID;

		case MESSAGE_EVENT_INVITE:   // ObjectID - event id, use_UserID - invited user (current)

			$event = new Event($message->ObjectID);

			switch (@$type)
			{
                case 'accept':
                    $event->addParticipant($targetID);
                    $message->MessageType = MESSAGE_EVENT_ACCEPTED;
                    $message->saveToDB();
                    break;

                case 'decline':
                    if ($targetID == $message->UserID)
                    {
                        $message->MessageType = MESSAGE_EVENT_REQUEST_DECLINED;
                    }
                    else
                    {
                        $message->MessageType = MESSAGE_EVENT_INVITE_DECLINED;
                    }

				default:
					$template->touchBlock('accept_decline');
			}
			break;

		case MESSAGE_GROUP_REQUEST: // ObjectID - group id, UserID - requesting user id
			$targetID = $message->UserID;

		case MESSAGE_GROUP_INVITE: // ObjectID - group id, use_UserID - invited user or GroupID or OrganizationID
									//  MessageFlags - official/unofficial
			$group = new Group($message->ObjectID);

			switch (@$type)
			{
                case 'accept':
                    if ($message->GroupID != -1)
                    {
                        $connectGroup = new Group($message->GroupID);
                        if ($connectGroup->GroupID != -1)
                        {
                            $connectGroup->addGroup($group->GroupID, $message->MessageFlags);
						}
					}
					else if ($message->OrganizationID != -1)
					{
                        $connectOrg = new Organization($message->OrganizationID);
                        if ($connectOrg->OrganizationID != -1)
                        {
                            $connectOrg->addGroup($group->GroupID, $message->MessageFlags);
						}
					}
					else
					{
						$group->addMember($targetID);
					}
                    $message->MessageType = MESSAGE_GROUP_ACCEPTED;
                    $message->saveToDB();
                    break;

                case 'decline':
                    $message->MessageType = MESSAGE_GROUP_DECLINED;
                    $message->saveToDB();
                    break;

				default:
					$template->touchBlock('accept_decline');
			}
			break;

            case MESSAGE_GROUP_ADMIN_INVITE: // ObjectID - group id, use_UserID - invited user to be admin


                $group = new Group($message->ObjectID);

                switch ($type)
                {
                    case 'accept':
                        $group->addAdmin($targetID);
                        $message->MessageType = MESSAGE_GROUP_ACCEPTED;
                        $message->saveToDB();
                        break;

                    case 'decline':
                        $message->MessageType = MESSAGE_GROUP_DECLINED;
                        $message->saveToDB();
                        break;

                    default:
                        $template->touchBlock('accept_decline');
                }
                break;

            case MESSAGE_ORGANIZATION_ADMIN_INVITE: // ObjectID - org id, use_UserID - invited user to be admin

                $org = new Organization($message->ObjectID);

                switch ($type)
                {
                    case 'accept':
                        $org->addAdmin($targetID);
                        $message->MessageType = MESSAGE_ORGANIZATION_ACCEPTED;
                        $message->saveToDB();
                        break;

                    case 'decline':
                        $message->MessageType = MESSAGE_ORGANIZATION_DECLINED;
                        $message->saveToDB();
                        break;

                    default:
                        $template->touchBlock('accept_decline');
                }
                break;

            case MESSAGE_MEMBER_COMMUNITY: // ObjectID - community id, UserID - from user id

                $org = new Community($message->ObjectID);

                $template->setVariable('TargetField', 'To');
                $template->setVariable('TargetName', $org->CommunityName);
                $template->parse('message_header');

                if ($org->isOwner($user->UserID))
                {
                    switch ($type)
                    {
                        case 'accept':
                            Message::sendCommunityUsersMessage($message->UserID, $org->CommunityID, $message->MessageSubject, $message->MessageText);
                            $message->MessageType = MESSAGE_COMMUNITY_ACCEPTED;
                            $message->saveToDB();
                            break;

                        case 'decline':
                            $message->MessageType = MESSAGE_COMMUNITY_DECLINED;
                            $message->saveToDB();
                            break;

                        default:
                            $template->touchBlock('accept_decline');
                    }
                }
                break;

            case MESSAGE_MEMBER_GROUP: // ObjectID - group id, UserID - from user id

                $org = new Group($message->ObjectID);

                $template->setVariable('TargetField', 'To');
                $template->setVariable('TargetName', $org->GroupName);
                $template->parse('message_header');

                $owner = $org->getOwner();
                if (get_class($owner) == 'Organization')
                {
                    $owner = $owner->getOwner();
                }
                if ($owner->UserID == $user->UserID)
                {
                    switch ($type)
                    {
                        case 'accept':
                            Message::sendGroupUsersMessage($message->UserID, $org->GroupID, $message->MessageSubject, $message->MessageText);
                            $message->MessageType = MESSAGE_GROUP_ACCEPTED;
                            $message->saveToDB();
                            break;

                        case 'decline':
                            $message->MessageType = MESSAGE_GROUP_DECLINED;
                            $message->saveToDB();
                            break;

                        default:
                            $template->touchBlock('accept_decline');
                    }
                }
                break;

            case MESSAGE_MEMBER_ORGANIZATION: // ObjectID - org id, UserID - from user id

                $org = new Organization($message->ObjectID);

                $template->setVariable('TargetField', 'To');
                $template->setVariable('TargetName', $org->OrganizationName);
                $template->parse('message_header');
                if ($org->isOwner($user->UserID))
                {
                    switch ($type)
                    {
                        case 'accept':
                            Message::sendOrganizationUsersMessage($message->UserID, $org->OrganizationID, $message->MessageSubject, $message->MessageText);
                            $message->MessageType = MESSAGE_ORGANIZATION_ACCEPTED;
                            $message->saveToDB();
                            break;

                        case 'decline':
                            $message->MessageType = MESSAGE_ORGANIZATION_DECLINED;
                            $message->saveToDB();
                            break;

                        default:
                            $template->touchBlock('accept_decline');
                    }
                }
                break;


            case MESSAGE_ORGANIZATION_REQUEST: // ObjectID - org id, UserID - requesting user id
                $targetID = $message->UserID;

            case MESSAGE_ORGANIZATION_INVITE:// ObjectID - group id, use_UserID - invited user or GroupID or OrganizationID
											 //  MessageFlags - official/unofficial
                $org = new Organization($message->ObjectID);

                switch ($type)
                {
                    case 'accept':
                        if ($message->GroupID != -1)
                        {
                        	$connect = new Group($message->GroupID);
                        	if ($connect->GroupID != -1)
                        	{
                        		$org->addGroup($connect->GroupID, $message->MessageFlags);
							}
						}
						else if ($message->OrganizationID != -1)
						{
                        	$connect = new Organization($message->OrganizationID);
                        	if ($connect->OrganizationID != -1)
                        	{
                        		$org->addOrganization($connect->OrganizationID, $message->MessageFlags);
							}
						}
						else
						{
							$org->addMember($targetID);
						}
                        $message->MessageType = MESSAGE_ORGANIZATION_ACCEPTED;
                        $message->saveToDB();
                        break;

                    case 'decline':
                        $message->MessageType = MESSAGE_ORGANIZATION_DECLINED;
                        $message->saveToDB();
                        break;

                    default:
                        $template->touchBlock('accept_decline');
                }
                break;

		case MESSAGE_USER_TAGGING_REQUEST:
			switch (@$type)
			{
				case 'accept':
					Tag::acceptUserTag($message->ObjectID);
					$message->MessageType = MESSAGE_ACCEPTED;
					$message->saveToDB();
					break;

				case 'decline':
					$message->MessageType = MESSAGE_DECLINED;
					$message->saveToDB();
					break;

				default:
					$template->touchBlock('accept_decline');
			}
			break;

		case MESSAGE_GROUP_TAGGING_REQUEST:
			switch (@$type)
			{
				case 'accept':
					Tag::acceptGroupTag($message->ObjectID);
					$message->MessageType = MESSAGE_ACCEPTED;
					$message->saveToDB();
					break;

				case 'decline':
					$message->MessageType = MESSAGE_DECLINED;
					$message->saveToDB();
					break;

				default:
					$template->touchBlock('accept_decline');
			}
			break;

		case MESSAGE_ORGANIZATION_TAGGING_REQUEST:
			switch (@$type)
			{
				case 'accept':
					Tag::acceptOrganizationTag($message->ObjectID);
					$message->MessageType = MESSAGE_ACCEPTED;
					$message->saveToDB();
					break;

				case 'decline':
					$message->MessageType = MESSAGE_DECLINED;
					$message->saveToDB();
					break;

				default:
					$template->touchBlock('accept_decline');
			}
			break;
	}

	switch ($message->MessageType)
	{
        case MESSAGE_BUDDY_ACCEPTED:
        case MESSAGE_EVENT_ACCEPTED:
        case MESSAGE_GROUP_ACCEPTED:
        case MESSAGE_ORGANIZATION_ACCEPTED:
		case MESSAGE_ACCEPTED:
			$template->touchBlock('accepted');
			break;

        case MESSAGE_BUDDY_DECLINED:
        case MESSAGE_EVENT_DECLINED:
        case MESSAGE_GROUP_DECLINED:
        case MESSAGE_ORGANIZATION_DECLINED:
		case MESSAGE_DECLINED:
			$template->touchBlock('declined');
			break;
	}

	$message_user = new User($message->UserID);
	$template->setGlobalVariable('UserID', $message->UserID);
	$template->setGlobalVariable('MessageID', $message->MessageID);

	if (!empty($message_user->UserPicture))
	{
    	$template->setVariable('img_avatar', $message_user->UserPicture);
    	$template->parse('avatar');
	}

	if (!empty($message->MessageFile))
	{
		$file_path = UPLOADED_FILES_PATH . '/' . $message->MessageFile;
		$template->setVariable('attFilePath', $file_path);
		$template->parse('attached_file');
	}

	$msgTxt = $message->MessageText;

	$msgTxt = str_replace("{html:open}", '<', $msgTxt);
	$msgTxt = str_replace("{html:close}", '>', $msgTxt);

	$msgTxt = nl2br($msgTxt);

	$template->setVariable('message_text', $msgTxt);

    $template->setVariable('messages_name', 'Message');
	if ($type != 'read')
	{
		$template->setVariable('message_subject', $message->MessageSubject);
		$template->setVariable('message_to', db_get_entity_display_name($message_user));
	}

	// sender & subject block
	$template->setVariable('MessageSubject', $message->MessageSubject);

	if ( ($message->MessageStatus==MESSAGE_STATUS_SENT) || ($message->MessageStatus==MESSAGE_STATUS_SENT_DELETED) )
	{
		$template->setVariable('TargetField', 'To');
		$template->setVariable('TargetName', $message->MessageRecipient);
        $template->parse('message_header');
	}
	else
	{
        if ($message->MessageStatus!=MESSAGE_STATUS_DELETED)
        {
            $template->touchBlock('answer2');
        }
        
		$sender = new User($message->UserID);
		$template->setVariable('TargetField', 'From');
		$template->setVariable('TargetName', $sender->UserFirstname . ' ' . $sender->UserLastname);
        $template->parse('message_header');
	}

	$template->parse('read_message');
}
else
{
    
	if (isset($_POST['messages']))
    {
    	if ($_POST['operation'] == 'mark_as_read')
    	{
    		$messages = $_POST['messages'];
    		foreach ($messages as $message_id => $value)
		    {
		        $message = new Message($message_id);

		        if ($message->UserID == $user->UserID || $message->use_UserID == $user->UserID)
		        {
		        	$message->MessageStatus = MESSAGE_STATUS_READ;
		        	$message->saveToDB();
		        }
		    }
    	}
    	elseif ($_POST['operation'] == 'delete')
    	{
    		$messages = $_POST['messages'];
    		foreach ($messages as $message_id => $value)
		    {
		        $message = new Message($message_id);

		        if ($message->UserID == $user->UserID || $message->use_UserID == $user->UserID)
		        {
		        	$message->moveToTrash();
		        }
		    }
    	}
    	else
    	{
	        $messages = $_POST['messages'];
		    foreach ($messages as $message_id => $value)
		    {
		        $message = new Message($message_id);

		        if ($message->UserID == $user->UserID || $message->use_UserID == $user->UserID)
		        {
	        		switch ($message->MessageStatus)
	        		{
	        			case MESSAGE_STATUS_SENT:
	        			case MESSAGE_STATUS_READ:
	        			case MESSAGE_STATUS_UNREAD:
	        				$message->moveToTrash();
	        				break;

	        			case MESSAGE_STATUS_DELETED:
	        			case MESSAGE_STATUS_SENT_DELETED:
	        				if ($_POST['operation'] == 'restore')
	        				{
	        					$message->RestoreMessage();
	        				}
	        				else
	        				{
	        					$message->delete();
	        				}
					}
				}
		    }
    	}
	}

    $messages_type = MESSAGE_ALL;
    switch ($_GET['filter'])
    {
		case MESSAGE_MEMBER_MEMBER:
		case MESSAGE_MEMBER_GROUP:
		case MESSAGE_MEMBER_COMMUNITY:
		case MESSAGE_MEMBER_ORGANIZATION:
		case MESSAGE_GROUP_INVITE:
		case MESSAGE_GROUP_REQUEST:
		case MESSAGE_ORGANIZATION_INVITE:
		case MESSAGE_ORGANIZATION_REQUEST:
		case MESSAGE_OFFICIAL:
		case MESSAGE_EVENT_INVITE:
		case MESSAGE_EVENT_REQUEST:
		case MESSAGE_RESUME_REQUEST:
		case MESSAGE_ACCEPTED:
		case MESSAGE_DECLINED:
			$messages_type = $_GET['filter'];
			$template->touchBlock('filter_' . $messages_type);
		break;
    }

    $messages = array();
    $type = $_GET['type'];

    switch (@$type)
	{
		case 'trash':
			$type = 'trash';
			$template->touchBlock('restore');
			$template->setVariable('messages_name', 'Trash');
			$messages = $user->getTrashMessages($messages_type);
			break;

		case 'sent':
			$type = 'sent';
			$template->setVariable('messages_name', 'Sent');
			$messages = $user->getSentMessages($messages_type);
			break;

		default:
			$type = 'inbox';
            $template->touchBlock('link_reply');
            $template->touchBlock('link_forward');
            $template->touchBlock('link_mark');
            
			$template->setVariable('messages_name', 'Inbox');
			$messages = $user->getInboxMessages($messages_type);
	}

	$template->touchBlock($type);
	$template->setVariable('type', $type);

	foreach ($messages as $message)
    {
        if ($type == 'sent')
        {
            $message_user = new User($message->use_UserID);
        }
        else
        {
            $message_user = new User($message->UserID);
        }
        $template->setVariable('user_id', $message_user->UserID);
        $template->setVariable('message_id', $message->MessageID);
        $template->setVariable('user_name', $message_user->UserFirstname . " " . $message_user->UserLastname);
        $template->setVariable('subject', $message->MessageSubject);
        $date = strtotime($message->MessageDate);
        $date = date('d/m/y', $date);
        $template->setVariable('date', $date);

        if ($message->MessageStatus == MESSAGE_STATUS_UNREAD)
        {
        	$template->setVariable('b1', '<b>');
        	$template->setVariable('b2', '</b>');
        }

        $template->parse('message');
    }
}

$type = $_GET['type'];

if (@$type == 'empty_sent')
{
    $user->emptySentMessages();
}

if (@$type == 'empty_trash')
{
	$user->emptyDeletedMessages();
}

if ($user->getMessagesCount($messages_type, MESSAGE_STATUS_SENT)>0)
{
    $template->touchBlock('empty_sent');
}

if ($user->getDeletedMessagesCount() > 0)
{
    $template->touchBlock('empty_trash');
}

$template->setVariable('unread_count', $user->getUnreadMessagesCount());

$template->touchBlock('fckeditor');
require_once('includes/application_bottom.php');